import os

import cv2
import glob
import numpy as np
import csv

from PyQt5 import QtWidgets, QtGui

from gui.pyqt5backend.guidesign import Ui_MainWindow

from filter.data import Data

class DesignerMainWindow(QtWidgets.QMainWindow, Ui_MainWindow, Data):
    """
    This is the Application class.
    It inherits Ui_MainWindow created in the Qt5Designer.
    It inherits the Data class to ensure access of image objects faster.
    App class DesignerMainWindow is called by main.py

    Flower Segmentation Tool
    * RGB image plot
    * crops images centers by percentage to original
    * to segment and visualize yellow flowers
    * adjust the minimum size of segmented areas
    * another visualization with red circles available
    * can be used on individual images or on images of a file directory

    Author: Anna Tenberg
    github: github.com/AnnaTe/arnica
    """

    def __init__(self, parent=None):
        # initialization of the superclass
        super(DesignerMainWindow, self).__init__(parent)
        # setup the GUI --> function generated by pyuic5
        self.setupUi(self)
        scriptDir = os.path.dirname(os.path.realpath(__file__))
        self.setWindowIcon(QtGui.QIcon(scriptDir + os.path.sep + 'arnica100.png'))
        # object from data class
        self.i = None
        self.perc = 100
        self.mins = None

        self.statusbar.showMessage('Ready')

        # File Menu signals
        self.actionSingle.triggered.connect(self.select_file)
        self.actionDirectory.triggered.connect(self.select_dir)
        self.actionClose.triggered.connect(self.close)

        self.lineEditImage.returnPressed.connect(self.select_file)

        # push button signals
        self.pbImageOpen.clicked.connect(self.select_file)
        self.pbUpdate.clicked.connect(self.update_graph)
        self.pbExport.clicked.connect(self.export_image)
        self.pbDirectoryOpen.clicked.connect(self.select_dir)
        self.pbDirectoryExport.clicked.connect(self.select_dir)
        self.pbRun.clicked.connect(self.run_export)

    def select_file(self):
        """opens a file select dialog and plots file"""
        self.statusbar.showMessage('Loading Image')

        # open file select dialog
        if self.sender().text() == 'Open Image' or self.sender().text() == 'Open':
            # open the dialog and get the selected file
            file = QtWidgets.QFileDialog.getOpenFileName(self, 'Select Image')
            self.lineEditImage.setText(file[0])
            self.tabWidget.setCurrentIndex(0)

        # try to plot input, raise Error if not an image
        try:
            self.initial_plot()
            self.perc = 100
            self.mins = None
        except:
            self.statusbar.showMessage('ERROR: File has to be an image. Try JPG or PNG Type.')

    def select_dir(self):
        """opens directory selection dialog"""
        directory = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select directory')
        if self.sender().text() == 'Export':
            self.lineEditDirOut.setText(directory)
        else:
            self.lineEditDirIn.setText(directory)
            self.tabWidget.setCurrentIndex(1)

    def parse_file(self):
        """ initiates image as an object of data class."""
        self.i = Data(self.lineEditImage.text())
        return self.i

    def plot(self, image):
        """ plots filtered image in Matplotlib canvas widget."""
        # clear the Axes
        self.mpl.canvas.ax.clear()
        # plot image
        self.mpl.canvas.ax.imshow(cv2.cvtColor(image, cv2.COLOR_BGR2RGB))
        self.mpl.canvas.ax.axis('off')
        # force an image redraw
        self.mpl.canvas.draw()
        self.statusbar.clearMessage()

    def initial_plot(self):
        """ plots first input image in Matplotlib canvas widget."""
        a = self.parse_file()
        self.plot(a.img)

    def update_graph(self):
        """ updates the plot in Matplotlib canvas widget. """
        self.statusbar.showMessage('update is running...')

        # collect values from user input
        percent= self.sbCrop.value()
        lowsize = self.sbBlob.value()

        if self.cbCircle.isChecked() == True:
            if percent != self.perc:
                self.i.filter(percent, lowsize)
                self.mins = lowsize
                self.perc = percent
            else:
                if lowsize != self.mins:
                    self.i.yellow(self.i.cropped, lowsize)
                    self.mins = lowsize
                else:
                    pass
            self.i.circleplot()
            self.plot(self.i.circle)
            self.statusbar.showMessage('{} Flowers counted.'.format(self.i.count))

        elif self.cbYellow.isChecked() == True:
            if percent != self.perc:
                self.i.filter(percent, lowsize)
                self.mins = lowsize
                self.perc = percent
            else:
                if lowsize != self.mins:
                    self.i.yellow(self.i.cropped, lowsize)
                    self.mins = lowsize
                else:
                    pass
            self.plot(self.i.blob)
            self.statusbar.showMessage('{} Flowers counted.'.format(self.i.count))

        else:
            if percent == self.perc:
                self.plot(self.i.cropped)
            else:
                self.perc = percent
                self.i.crop(self.perc)
                self.plot(self.i.cropped)

    def export_image(self):
        """Exports the plotted image with filedialog."""
        self.statusbar.showMessage('Export is running...')

        # collect values from user input
        percent= self.sbCrop.value()
        lowsize = self.sbBlob.value()
        try:
            saveas = QtWidgets.QFileDialog.getSaveFileName(self, 'Save as')[0]
        except:
            self.statusbar.showMessage('Export failed, try again.')
        if self.cbCircle.isChecked() == True:
            if percent != self.perc:
                self.i.filter(percent, lowsize)
                self.mins = lowsize
                self.perc = percent
            else:
                if lowsize != self.mins:
                    self.i.yellow(self.i.cropped, lowsize)
                    self.mins = lowsize
            self.i.circleplot()
            self.plot(self.i.circle)
            self.statusbar.showMessage('{} Flowers counted.'.format(self.i.count))
            try:
                cv2.imwrite(saveas, self.i.circle)
            except:
                self.statusbar.showMessage("ERROR: Not a valid file name. File type has to be JPG or PNG.")

        elif self.cbYellow.isChecked() == True:
            if percent != self.perc:
                self.i.filter(percent, lowsize)
                self.mins = lowsize
                self.perc = percent
            else:
                if lowsize != self.mins:
                    self.i.yellow(self.i.cropped, lowsize)
                    self.mins = lowsize
            try:
                cv2.imwrite(saveas, self.i.blob)
            except:
                self.statusbar.showMessage("ERROR: Not a valid file name. File type has to be JPG or PNG.")
            else:
                self.statusbar.clearMessage()
        else:
            if percent == self.perc:
                pass
            else:
                self.perc = percent
                self.i.crop(self.perc)
            try:
                cv2.imwrite(saveas, self.i.cropped)
            except:
                self.statusbar.showMessage("ERROR: Not a valid file name. File type has to be JPG or PNG.")
            else:
                self.statusbar.clearMessage()

    def run_export(self):
        """runs process for all images of directory and exports into output directory"""
        self.statusbar.showMessage('Export is running...')

        # all path of files from directory
        path = self.lineEditDirIn.text() + '/*'
        paths = glob.glob(path)
        imagetypes = ['png','jpg', 'jpeg', 'bmp', 'dib', 'jpe', 'jp2', 'webp', 'pbm','pgm','ppm', 'sr', 'ras', 'tiff', 'tif']
        for file in paths:
            if '.' not in file:
                paths.remove(file)
        for file in paths:
            if file.split('.')[-1].casefold() not in imagetypes:
                paths.remove(file)

        # create output directory if necessary
        outputdir = self.lineEditDirOut.text() + '/'
        os.makedirs(outputdir, exist_ok= True)

        # set value of progressbar
        self.completed = 0
        self.total = len(paths)

        # collect values from user input
        percent = self.sbCropDir.value()
        lowsize = self.sbBlobDir.value()

        csvname = outputdir + 'flowercount.csv'

        with open(csvname, 'w') as csvfile:
            filewriter = csv.writer(csvfile, delimiter=',', quotechar='|', quoting=csv.QUOTE_MINIMAL)
            for imagepath in paths:
                self.i = Data(imagepath)
                if self.cbCirclesDir.isChecked() == True:
                    try:
                        self.i.filter(percent, lowsize)
                        self.i.circleplot()
                    except:
                        self.statusbar.showMessage("ERROR: Images in import directory not found. Try again.")
                    outpath = outputdir + self.i.name + 'circ' + str(lowsize) + '.png'

                    filewriter.writerow([self.i.name, self.i.count])
                    cv2.imwrite(outpath, self.i.circle)
                    self.completed += 1
                    self.statusbar.showMessage("Running: {} of {} images exported.".format(self.completed, self.total))
                elif self.cbYellowDir.isChecked() == True:
                    try:
                        self.i.filter(percent, lowsize)
                    except:
                        self.statusbar.showMessage("ERROR: Images in import directory not found. Try again.")

                    outpath = outputdir + self.i.name + 'seg' + str(lowsize) + '.png'
                    filewriter.writerow([self.i.name, self.i.count])
                    cv2.imwrite(outpath, self.i.blob)
                    self.completed += 1
                    self.statusbar.showMessage("Running: {} of {} images exported.".format(self.completed, self.total))
                else:
                    try:
                        self.i.crop(percent)
                    except:
                        self.statusbar.showMessage("ERROR: Images in import directory not found. Try again.")
                    outpath = outputdir + self.i.name + 'crop' + str(percent) + '.png'
                    cv2.imwrite(outpath, self.i.cropped)
                    self.completed += 1
                    self.statusbar.showMessage("Running: {} of {} images exported.".format(self.completed, self.total))

        self.statusbar.showMessage("Process finished.", 500)

